package com.loafle.commons.rpc.protocol.json;

import java.io.IOException;

import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.protocol.RPCServerCodec;
import com.loafle.commons.rpc.protocol.RPCServerRequestCodec;

import org.codehaus.jackson.map.DeserializationConfig;
import org.codehaus.jackson.map.ObjectMapper;

/**
 * JSONRPCServerCodec
 */
public class JSONRPCServerCodec implements RPCServerCodec {
  private ObjectMapper objectMapper;

  public JSONRPCServerCodec() {
    this.objectMapper = new ObjectMapper();
    this.objectMapper.configure(DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES, false);
  }

  public JSONRPCServerCodec(ObjectMapper objectMapper) {
    this.objectMapper = objectMapper;
  }

  public void setObjectMapper(ObjectMapper objectMapper) {
    this.objectMapper = objectMapper;
  }

  public RPCServerRequestCodec request(byte[] buff) throws RPCException {
    return new JSONRPCServerRequestCodec(this.objectMapper, buff);
  }

  public byte[] notification(String method, Object... params) throws RPCException {
    JSONServerNotification notification = new JSONServerNotification(method, params);
    JSONServerResponse response = new JSONServerResponse(JSONRPC.version, notification, null);

    try {
      String json = this.objectMapper.writeValueAsString(response);
      return json.getBytes();
    } catch (IOException e) {
      throw new RPCException("cannot marsharling", e);
    }
  }
}
